function mapGen(ecks,wie,quality)
	worldTable = {}
	worldTable.units = {}
	worldTable.buildings = {}
	worldTable.vegetation = {}
	teamTable = {}
	if not ecks then ecks = 30 end; if not wie then wie = 60 end
	worldSize = {ecks, wie}
	worldSize.limits = { --FOR LIMITING THE VIEW
		(ecks/2),
		(wie/2)
		}

	--nodes for generation
	local distnodex = math.random(3,7)
	local distnodey = 10 - distnodex + math.random(-2,2)
	local avgnodedist = (distnodex + distnodey) / 2
	local pythagnodedist = math.sqrt(distnodex^2 + distnodey^2)
	--limit the amount of node comparisons
	local countlimit = 20 * quality or 20
	--upper and lower limit vars for random vs later on
	local upperlim = 0
	local lowerlim = 0
	--generate the worldtable
	local count = 0
	print("+----[WORLDGENERATION]-- -");print("|  NODES AND TABLE")
	for x = 0, ecks - 1 do
		for y = 0, wie - 1 do
			count = count + 1
			--first tile medial
			if #worldTable < 1 then
				createTile({x, y}, 0, math.random(8, 16), math.random(2, #floormat), 1, count, "NORMAL")
			--if a node then semi-random generation
			elseif math.floor(x/distnodex) == x/distnodex and math.floor(y/distnodey) == y/distnodey then
				createTile({x, y}, worldTable[#worldTable].alt + math.random(-20,20), math.random(8, 16), math.random(2, #floormat), 1, count, "NORMAL")
			else --if not then use relative generation.
				if math.random(2) == 1 and x > 1 then
					createTile({x, y}, worldTable[#worldTable - ecks + 1].alt + math.random(-5,5), math.random(8, 16), math.random(2, #floormat), 1, count, "NORMAL")
				else
					createTile({x, y}, worldTable[#worldTable].alt + math.random(-5,5), math.random(8, 16), math.random(2, #floormat), 1, count, "NORMAL")
				end
			end
		end
	end
	
	print("|  NEIGHBOUR ASSIGNMENT") --fixed
	for k,v in ipairs(worldTable) do
		v:getNeighbours()
	end
	
	--smooth it off. less crazy jaggies
	print("|  BASIC ALITUDE AVERAGING AND CLAMPING")
	for k,v in ipairs(worldTable) do
		halfAvgNeighbours(v)
		if v.alt > 40 then
			v.alt = 40
		elseif v.alt < -50 then
			v.alt = -50
		end
		if v.alt > upperlim then
			upperlim = v.alt
		elseif v.alt < lowerlim then
			lowerlim = v.alt
		end
	end
	
	print("|  STRATIFICATION")
	for k,v in ipairs(worldTable) do
		if v.alt > 30 and math.random(6) > 1 then
			v.type = math.random(#floormat - 1, #floormat)
		elseif v.alt > 20 and math.random(6) > 1 then
			v.type = 2
		elseif v.alt > 15 and math.random(6) > 1 then
			v.type = 3
		elseif v.alt > 10 and math.random(6) > 1 then
			v.type = 4
		elseif v.alt > -5 and math.random(6) > 1 then
			v.type = 5
		elseif v.alt > -10 and math.random(6) > 1 then	
			v.type = math.random(2, #floormat)
		elseif v.alt > -20 and math.random(6) > 1 then
			v.type = 5
		elseif math.random(2) == 1 then
			v.type = #floormat
		end
	end
	
	----------------------------------------------------------------------
	--now the fun happens
	----------------------------------------------------------------------
	
	--fault line generation --
	print("|  FAULT GENERATION")
	print("|   -NOISE POINTS")		-- create some random points for vertical noise
	local noisepoints = {}
	local needsaveraging = {}
	for k,v in ipairs(worldTable) do
		if math.random(#worldTable / 4) == 1 then
			for k2,v2 in ipairs(v.neighbours) do
				v2.alt = math.random(-2, 2)
				v2.alt = v2.alt * 20
				tabRadius(v, worldTable, math.random(2,5), needsaveraging)
			end
			if math.random(2) == 1 then
				v.alt = 45
			else
				v.alt = -45
			end
			table.insert(noisepoints, v)
		end
	end
	--print("|   ["..#noisepoints.."]") --for number printing
	print("|   -CONNECTION")
	for i = 1, #noisepoints do
		aStarBetween(noisepoints[i], noisepoints[math.random(#noisepoints)], worldTable, needsaveraging)
	end
	print("|   -CONSTRUCTION")
	for k,v in ipairs(needsaveraging) do
		v.alt = math.random(-4,4)
		v.alt = v.alt * 8
		for key, neighbour in ipairs(v.neighbours) do
			neighbour.type = math.random(#floormat - 1,#floormat)
			halfAvgNeighbours(neighbour)
		end
		v.type = #floormat
	end
	for k,v in ipairs(needsaveraging) do
		avgNeighbours(v)
	end
	needsaveraging = {}
	noisepoints = {}
	--end fault generation --
	
	--channel the area	
	print("|  CHANNELLING")
	local count = 1
	for k,v in ipairs(worldTable) do
		if math.random(10) == 1 and v.alt < 3 and v.alt > -3 then
			addChannel(v)
			count = count + 1
		end
		if count > ecks / 5 then break end
	end
	
	--construct cliffs
	print("|  CLIFF PROPAGATION")
	for k,v in ipairs(worldTable) do
		for k2,v2 in ipairs(v.neighbours) do
			if v2.alt > v.alt + 2 then
				if math.random(7) > 2 then
					v2.alt = v2.alt + math.random(1,4)
					v.alt = v.alt - math.random(1,4)
				else
					v2.alt = v2.alt + math.random(3,5)
					v.alt = v.alt - math.random(1,3)
					v2.type = #floormat
					v.type = #floormat
				end
			end
		end
	end
	
	----------------------------------------------------------------------
	--area designation - deserts, steppes, wtvr.
	print("|  AREA DESIGNATION")
	--create an area to be checked
	steppes = {}
	desert = {}
	plains = {}
	rockland = {}
	for k,tile in ipairs(worldTable) do
		if not tile.taken then
			if math.random(#steppes / 2 + 2) == 1 then
				xTabRadius(tile, worldTable, math.random(4,10), steppes)
				tile.taken = true
			elseif math.random(#desert / 2 + 2) == 1 then
				xTabRadius(tile, worldTable, math.random(10,20), desert)
				tile.taken = true
			elseif math.random(#plains / 2 + 2) == 1 then
				xTabRadius(tile, worldTable, math.random(10,20), plains)
				tile.taken = true
			elseif math.random(#rockland / 2 + 2) == 1 then
				xTabRadius(tile, worldTable, math.random(4,10), rockland)
				tile.taken = true
			end
		end
	end
	
	--STEPPES
	if #steppes > 0 then
		print("|  HIGHLAND AND VALLEY CREATION")
		--grab a random line or so to add :D
		for i = 1, math.random(1,math.sqrt(#worldTable) / 5) do
			aStarBetween(steppes[math.random(#steppes)], worldTable[math.random(#worldTable)], worldTable, steppes)
		end
		--get the midpoint
		local altavg = 0
		for k,v in ipairs(steppes) do
			altavg = altavg + v.alt
		end
		altavg = altavg / #steppes
		local highs = {} --create the highs table
		for k,v in ipairs(steppes) do
			if v.alt < altavg then
				table.insert(highs, v)
				v.alt = -60 + math.random(-8,8)
				v.type = #floormat
				v.tag = "HIGHLANDS"
			end
		end
		
		local lows = {} --create the lows table
		for k,v in ipairs(steppes) do
			if v.alt > altavg then
				table.insert(lows, v)
				v.alt = 40 + math.random(-3,3)
				v.type = 4
				v.tag = "LOWLANDS"
			end
		end
		
		for k,v in ipairs(highs) do
			for k2,v2 in ipairs(v.neighbours) do
				if v2.tag ~= "HIGHLANDS" then
					v2.alt = v.alt + math.random(-4,10)
					halfAvgNeighbours(v2)
				end
			end
			if math.random(#highs) == 1 then
				v.alt = -100 --mountain
			end
			avgNeighbours(v)
		end
		
		for k,v in ipairs(lows) do
			for k2,v2 in ipairs(v.neighbours) do
				if v2.tag ~= "LOWLANDS" then
					v2.alt = v.alt - math.random(-10,4)
					halfAvgNeighbours(v2)
				end
			end
			avgNeighbours(v)
		end
		
		--clear the tables
		steppes = {}
		gradiate = {}
		lows = {}
		highs = {}
	else
		print("|  NO HIGHLANDS")
	end
	----------------------------------------------------------------------

	
	--FLATLANDS
	if #plains > 0 then
		print("|  FLATLAND CREATION AND EMPHASIS")
		for i = 1, math.random(1,math.sqrt(#worldTable) / 5) do
			aStarBetween(plains[math.random(#plains)], worldTable[math.random(#worldTable)], worldTable, plains)
		end
		print("|  CREATION")
		--get the midpoint
		local altavg = 0
		for k,v in ipairs(plains) do
			altavg = altavg + v.alt
			v.tag = "PLAINS"
		end
		altavg = altavg / #plains
		print("|  EMPHASIS")
		for k,v in ipairs(plains) do
			v.alt = altavg
			for k2,v2 in ipairs(v.neighbours) do
				if v.tag ~= "PLAINS" then
					halfAvgNeighbours(v2)
				end
			end
			v.type = 7
		end
	else
		print("|  NO PLAINS")
	end
	
	--rockland alteration
	if #rockland > 0 then
		for i = 1, math.random(1,math.sqrt(#worldTable) / 5) do
			aStarBetween(rockland[math.random(#rockland)], worldTable[math.random(#worldTable)], worldTable, rockland)
		end
		for k,v in ipairs(rockland) do
			if v.tag == "NORMAL" then
				v.alt = v.alt + math.random(-3,3)
				v.type = #floormat
			end
		end
	end
	----------------------------------------------------------------------
	
	--desert alteration
	if #desert > 0 then
		print("|  GENERATE DESERT")
		for k,v in ipairs(desert) do
			if v.tag == "NORMAL" then
				v.type = #floormat - 1
				v.tag = "DESERT"
			end
		end
	else
		print("|  NO DESERT")
	end
	----------------------------------------------------------------------
	
	--limitation of cliff gradient.
	for k,v in ipairs(worldTable) do
		local a = v.alt
		local b = v.alt
		for k2,v2 in ipairs(v.neighbours) do
			if v2.alt > a then
				a = v2.alt
			elseif v2.alt < b then
				b = v2.alt
			end
		end
		if a - b > 20 or v.tag == "NORMAL" then
			avgRadius(worldTable, v, 1, pythagnodedist, countlimit, upperlim, lowerlim)
		elseif a - b > 15 then
			avgNeighbours(v)
		elseif a - b > 10 then
			halfAvgNeighbours(v)
		end
	end
	
	--do some erosion stuff, based on neighbours	
	print("|  EROSION")
	for k,v in ipairs(worldTable) do
		if math.random(2) == 1 then
			for k2,v2 in ipairs(v.neighbours) do
				if v2.alt < v.alt then
					v.alt = v.alt + .7
				else
					v.alt = v.alt - .5
				end
			end
			v.alt = round(v.alt)
		end
	end
	
	--smooth off all the crazy bits
	print("|  REAVERAGING")
	for k,v in ipairs(worldTable) do
		avgRadius(worldTable, v, .5, 5, countlimit, upperlim, lowerlim)
	end
	
	--water stuff here --CURRENTLY COMPLETELY BUGGERED.
	
	
	--make the type stuff less patchy, but more rugged at the edges
	print("|  TYPE REAVERAGING")
	print("|   BLOTTING")
	for k,v in ipairs(worldTable) do
		reblotting(v)
	end
	
	print("|   SMOOTHING")
	for k,v in ipairs(worldTable) do
		tAvgRadius(worldTable, v, 2, countlimit)
		tAvgNeighbours(v)
	end
	
	print("|  TAG FINALISATION AND HOUSEKEEPING")
	--cliff tagging & numbering
	for k,v in ipairs(worldTable) do
		local limit = 15
		for k2,v2 in ipairs(v.neighbours) do
			if (v2.alt - v.alt) >= limit then
				v.tag = "CLIFF"
				v.type = #floormat
				v.cliffnum = round((v2.alt - v.alt) / 16) + 1
				limit = (v2.alt - v.alt)
			end
		end
		if v.type == 1 then --water alt limiting
			v.alt = 40
		end
	end
	
	print("|  KIND AND COLOUR ASSIGNMENT")
	for k,v in ipairs(worldTable) do
		v.kind = math.random(#floormat[v.type])
		v.ckind = math.random(#cliffmat[v.type])
		--print("TYPE: "..v.type.."|KIND: "..v.kind.."|CKIND: "..v.ckind.."|ALT: "..v.alt) --readout
		if v.alt > 40 then
			v.colour = 128
		elseif v.alt < -40 then
			v.colour = 255
		else
			v.colour = (128 + (-v.alt * 3) + 255) / 2
		end
	end
	--end of terrain gen!
	
	
	--start the vegetation generation
	print("+----[VEGETATIONGENERATION]-- -");print("|  LOCATION ARRANGEMENT")
	worldTable.vegetation = {}
	for k,v in ipairs(worldTable) do
		if v.type >= 3 and v.type <= 6 and math.random(2) == 1 and v.tag ~= "PLAINS" then
			createTree(v, math.random(1,2))
			v.tag = "FOREST"
		elseif v.tag == "PLAINS" and math.random(10) == 1 then
			createTree(v, 1)
		end
	end
	print("|  COLLABORATIVE DECAY")
	for k,v in ipairs(worldTable.vegetation) do
		if counttreeneighbours(v) > 1 then
			v.size = getlps(v) - math.random(getlps(v) - 1)
		else
			v:destroy()
		end
	end
	
	--start the vegetation generation
	print("+----[MINERALGENERATION]-- -");print("|  LOCATION ARRANGEMENT")
	for k,v in ipairs(worldTable) do
		if v.type == #floormat and v.tag ~= "CLIFF" and math.random(3) == 1 then
			createMineral(v)
			v.tag = "MINERAL"
		end
	end
	
	--start the civilisation generation.
	print("+----[CIVGEN]-- -");print("|  LOCATION ARRANGEMENT")
	--how many starting civilisations?
	worldTable.civCount = math.random(2,math.log(#worldTable/2, 30))
	worldTable.units = {}
	local count = 0
	for k,v in ipairs(worldTable) do
		if count < worldTable.civCount then
			if math.random(math.log(#worldTable, 20)) == 1 and #v.occupants == 0 and v.tag ~= "CLIFF" and checkCivRad(v, worldTable) then
				--CIV HUB POINT!
				if v.tag == "PLAINS" then --hoomins
					count = count + 1
					--for limiting units
					local unitcount = 1
					local buildingcount = 1
					createTeam("MAKE A GRAMMAR DO IT", "PLAINS", {v.pos[1], v.pos[2]}, count)
					createBuilding("HUB", "ECONOMIC", 16, count, v)
					for k2,v2 in ipairs(v.neighbours) do
						for k3, v3 in ipairs(v2.neighbours) do
							v3.tag = "PLAINS"
							v3.type = 7
							if #v3.occupants == 0 then
								if math.random(1,unitcount + 1) <= 5 then
									createUnit("unit_"..unitcount.."_"..count, "ECO", 1, count, v3, math.random(8,22) / 10, 5)
									unitcount = unitcount + 1
								elseif math.random(buildingcount + 8) == 1 and teamTable[count].wood >= 10 then
									createBuilding("HOUSE", "ECONOMIC", 10, count, v3)
									teamTable[count].wood = teamTable[count].wood - 10
									buildingcount = buildingcount + 1
								end
							end
						end
						v2.tag = "PLAINS"
						v2.type = 7
						if #v2.occupants == 0 then
							if math.random(buildingcount + 5) == 1 and teamTable[count].wood >= 10 then
								createBuilding("HOUSE", "ECONOMIC", 10, count, v2)
								teamTable[count].wood = teamTable[count].wood - 10
								buildingcount = buildingcount + 1
							end
						end
					end
					v.tag = "HUB"
				elseif v.tag == "FOREST" then --skulkies
					count = count + 1
					--for limiting units
					local unitcount = 1
					local buildingcount = 1
					createTeam("MAKE A GRAMMAR DO IT", "FOREST", {v.pos[1], v.pos[2]}, count)
					createBuilding("HUB", "ECONOMIC", 16, count, v)
					for k2,v2 in ipairs(v.neighbours) do
						for k3, v3 in ipairs(v2.neighbours) do
							v3.tag = "FOREST"
							v3.type = math.random(3,5)
							if #v3.occupants == 0 then
								if math.random(1,unitcount + 1) <= 5 then
									createUnit("unit_"..unitcount.."_"..count, "ECO", 1, count, v3, math.random(8,22) / 10, 5)
									unitcount = unitcount + 1
								elseif math.random(buildingcount + 8) == 1 and teamTable[count].wood >= 10 then
									createBuilding("HOUSE", "ECONOMIC", 10, count, v3)
									teamTable[count].wood = teamTable[count].wood - 10
									buildingcount = buildingcount + 1
								end
							end
						end
						v2.tag = "FOREST"
						v2.type = math.random(3,5)
						if #v2.occupants == 0 then
							if math.random(buildingcount + 5) == 1 and teamTable[count].wood >= 10 then
								createBuilding("HOUSE", "ECONOMIC", 10, count, v2)
								teamTable[count].wood = teamTable[count].wood - 10
								buildingcount = buildingcount + 1
							end
						end
					end
					v.tag = "HUB"
				elseif v.tag == "MINERALS" or v.tag == "HIGHLANDS" then --bigguns
					count = count + 1
					--for limiting units
					local unitcount = 1
					local buildingcount = 1
					createTeam("MAKE A GRAMMAR DO IT", "EARTH", {v.pos[1], v.pos[2]}, count)
					createBuilding("HUB", "ECONOMIC", 16, count, v)
					for k2,v2 in ipairs(v.neighbours) do
						for k3, v3 in ipairs(v2.neighbours) do
							if #v3.occupants == 0 then
								if math.random(1,unitcount + 1) <= 5 then
									createUnit("unit_"..unitcount.."_"..count, "ECO", 1, count, v3, math.random(8,22) / 10, 5)
									unitcount = unitcount + 1
								elseif math.random(buildingcount + 8) == 1 and teamTable[count].wood >= 10 then
									createBuilding("HOUSE", "ECONOMIC", 10, count, v3)
									teamTable[count].wood = teamTable[count].wood - 10
									buildingcount = buildingcount + 1
								end
							end
						end
						if #v2.occupants == 0 then
							if math.random(buildingcount + 5) == 1 and teamTable[count].wood >= 10 then
								createBuilding("HOUSE", "ECONOMIC", 10, count, v2)
								teamTable[count].wood = teamTable[count].wood - 10
								buildingcount = buildingcount + 1
							end
						end
					end
					v.tag = "HUB"
				elseif v.tag == "WATER" then --sqihd.
					count = count + 1
					--for limiting units
					local unitcount = 1
					local buildingcount = 1
					createTeam("MAKE A GRAMMAR DO IT", "WATER", {v.pos[1], v.pos[2]}, count)
					createBuilding("HUB", "ECONOMIC", 16, count, v)
					for k2,v2 in ipairs(v.neighbours) do
						for k3, v3 in ipairs(v2.neighbours) do
							v3.tag = "WATER"
							v3.type = 1
							if #v3.occupants == 0 then
								if math.random(1,unitcount + 1) <= 5 then
									createUnit("unit_"..unitcount.."_"..count, "ECO", 1, count, v3, math.random(8,22) / 10, 5)
									unitcount = unitcount + 1
								elseif math.random(buildingcount + 8) == 1 and teamTable[count].wood >= 10 then
									createBuilding("HOUSE", "ECONOMIC", 10, count, v3)
									teamTable[count].wood = teamTable[count].wood - 10
									buildingcount = buildingcount + 1
								end
							end
						end
						v2.tag = "WATER"
						v2.type = 1
						if #v2.occupants == 0 then
							if math.random(buildingcount + 5) == 1 and teamTable[count].wood >= 10 then
								createBuilding("HOUSE", "ECONOMIC", 10, count, v2)
								teamTable[count].wood = teamTable[count].wood - 10
								buildingcount = buildingcount + 1
							end
						end
					end
					v.tag = "HUB"
				elseif math.random(2) == 1 then --RANDOM WOO, lower chance though.
					count = count + 1
					--for limiting units
					local unitcount = 1
					local buildingcount = 1
					createTeam("MAKE A GRAMMAR DO IT", "PLAINS", {v.pos[1], v.pos[2]}, count)
					createBuilding("HUB", "ECONOMIC", 16, count, v)
					for k2,v2 in ipairs(v.neighbours) do
						for k3, v3 in ipairs(v2.neighbours) do
							if #v3.occupants == 0 then
								if math.random(1,unitcount + 1) <= 5 then
									createUnit("unit_"..unitcount.."_"..count, "ECO", 1, count, v3, math.random(8,22) / 10, 5)
									unitcount = unitcount + 1
								elseif math.random(buildingcount + 8) == 1 and teamTable[count].wood >= 10 then
									createBuilding("HOUSE", "ECONOMIC", 10, count, v3)
									teamTable[count].wood = teamTable[count].wood - 10
									buildingcount = buildingcount + 1
								end
							end
						end
						if #v2.occupants == 0 then
							if math.random(buildingcount + 5) == 1 and teamTable[count].wood >= 10 then
								createBuilding("HOUSE", "ECONOMIC", 10, count, v2)
								teamTable[count].wood = teamTable[count].wood - 10
								buildingcount = buildingcount + 1
							end
						end
					end
					v.tag = "HUB"
				end
			end
		else break end
	end
	worldTable.civCount = count -- just incase we didn't get as many as we wanted.
	
	--give back some stats!
	print("+----[WORLD STATISTICS]-- -")
	--trees & minerals
	local numtrees = 0; local numminerals = 0;
	--land types
	local numwater = 0; local numplains = 0; local numhighlands = 0; local numlowlands = 0; local numdesert = 0;
	local numforest = 0; local numminerals = 0 local numcliffs = 0; local numnormal = 0;
	--buildings and units
	local numbuildings = 0; local numunits = 0;
	for k,v in ipairs(worldTable) do
		for k2,v2 in ipairs(v.occupants) do
			if v2.class == "RESOURCE_TREE" then
				numtrees = numtrees + 1
			elseif v2.class == "RESOURCE_MINERAL" then
				numminerals = numminerals + 1
			elseif v2.class == "BUILDING" then
				numbuildings = numbuildings + 1
			elseif v2.class == "UNIT" then
				numunits = numunits + 1
			end
		end
		if v.tag == "PLAINS" then
			numplains = numplains + 1
		elseif v.tag == "WATER" then
			numwater = numwater + 1
		elseif v.tag == "HIGHLANDS" then
			numhighlands = numhighlands + 1
		elseif v.tag == "LOWLANDS" then
			numlowlands = numlowlands + 1
		elseif v.tag == "DESERT" then
			numdesert = numdesert + 1
		elseif v.tag == "FOREST" then
			numforest = numforest + 1
		elseif v.tag == "MINERAL" then
			numminerals = numminerals + 1
		elseif v.tag == "CLIFF" then
			numcliffs = numcliffs + 1
		else
			numnormal = numnormal + 1
		end
	end
	print("|  Number of trees:    "..numtrees)
	print("|  Number of minerals: "..numminerals)
	print("|  LAND DISTRIBUTION STATS:")
	print("|   Water:    "..tostring((numwater/#worldTable)*100).."%")
	print("|   Plains:   "..tostring((numplains/#worldTable)*100).."%")
	print("|   Highland: "..tostring((numhighlands/#worldTable)*100).."%")
	print("|   Lowland:  "..tostring((numlowlands/#worldTable)*100).."%")
	print("|   Desert:   "..tostring((numdesert/#worldTable)*100).."%")
	print("|   Forest:   "..tostring((numforest/#worldTable)*100).."%")
	print("|   Mineral:  "..tostring((numminerals/#worldTable)*100).."%")
	print("|   Cliffs:   "..tostring((numcliffs/#worldTable)*100).."%")
	print("|   (other):  "..tostring((numnormal/#worldTable)*100).."%")
	print("|  CIVILISATION REPORT:")
	print("|   #Civiliations: "..#teamTable)
	print("|   #Units:        "..#worldTable.units)
	print("|   #Buildings:    "..#worldTable.buildings)
	print("+----[DONE]-- -")
	return true
end

function avgNeighbours(t)
	local valt = 0
	for k,v in ipairs(t.neighbours) do
		valt = valt + v.alt
	end
	t.alt = round(valt / #t.neighbours)
end

function halfAvgNeighbours(t)
	local valt = 0
	for k,v in ipairs(t.neighbours) do
		valt = valt + v.alt
	end
	t.alt = round(((valt / #t.neighbours) + t.alt) / 2)
end

function avgRadius(wt, t, s, pnd, cl, ul, ll)
	local valt = 0
	local count = 0
	local UL = ul or 10
	local LL = ll or -10
	for k, v in ipairs(wt) do
		if v ~= t then
			if math.sqrt((t.pos[1] - v.pos[1]) ^ 2 + (t.pos[2] - v.pos[2]) ^ 2) < pnd / 2 then
				valt = valt + v.alt
				count = count + 1; if count >= cl / s then break end
			end
		end
	end
	if count > 3 then
		t.alt = round(valt / count)
	else
		t.alt = math.random(LL, UL)
	end
	return true
end

function checkCivRad(t, wt)
	for k,v in ipairs(wt) do
		if v.tag == "HUB" then
			if math.sqrt((v.pos[1] - t.pos[1]) ^ 2 + (v.pos[2] - t.pos[2]) ^ 2) < math.sqrt(#worldTable) / 2 then
				return false
			end
		end
	end
	return true
end

function tabRadius(t, wt, r, tab)
	for k,v in ipairs(wt) do
		if v ~= t and math.sqrt((v.pos[1] - t.pos[1]) ^2 + (v.pos[2] - t.pos[2]) ^2) < r then
			table.insert(tab, v)
		end
	end
end

function xTabRadius(t, wt, r, tab)
	for k,v in ipairs(wt) do
		if v ~= t and math.sqrt((v.pos[1] - t.pos[1]) ^2 + (v.pos[2] - t.pos[2]) ^2) < r and not v.taken then 
			table.insert(tab, v)
			v.taken = true
		end
	end
end

function tAvgNeighbours(t)
	local vtype = t.type
	for k, v in ipairs(t.neighbours) do
		vtype = vtype + v.type
	end
	t.type = round(vtype / (#t.neighbours + 1))
end

function tAvgRadius(wt, t, r, cl)
	local vtype = t.type
	local count = 0
	for k, v in ipairs(wt) do
		if v ~= t then
			if math.sqrt((t.pos[1] - v.pos[1]) ^ 2 + (t.pos[2] - v.pos[2]) ^ 2) < r then
				vtype = vtype + v.type
				count = count + 1; if count >= cl then break end
			end
		end
	end
	if count > 3 then
		t.type = round(vtype / (count + 1))
	end
end

function reblotting(t)
	local lonely = true
	local tab = {}
	for k,v in ipairs(t.neighbours) do
		if v.type == t.type then
			lonely = false break
		else table.insert(tab, v)
		end
	end
	if lonely then
		if math.random(2) == 1 then
			t.type = t.neighbours[1].type
		else
			local tcount = 0
			for k,v in ipairs(tab) do
				tcount = tcount + v.type
			end
			t.type = round(tcount / #tab)
		end
	end
end

function addChannel(t)
	local active = t
	local level = t.alt
	local count = 0
	while not done do
		for k, v in ipairs(active.neighbours) do
			if math.random(2) == 1 then
				v.alt = level + math.random(-1, 1)
				if math.random(2) == 1 then
					active = v
					v.tag = "CLIFF"
					for k2,v2 in ipairs(v.neighbours) do
						avgNeighbours(v2)
					end
					level = v.alt break
				end
				count = count + 1
			end
		end
		if count > 30 or math.random(30) == 1 then done = true end
	end
end

function aStarBetween(t1, t2, wt, st)
	for k,v in ipairs(wt) do
		v.adist = round(math.sqrt( math.abs(t2.pos[1] - v.pos[1]) ^ 2 + math.abs(t2.pos[2] - v.pos[2]) ^ 2 ))
	end
	local active = t1
	if not active.adist then
		active.adist = round(math.sqrt( math.abs(t2.pos[1] - active.pos[1]) ^ 2 + math.abs(t2.pos[2] - active.pos[2]) ^ 2 )) end
	local count = 1
	while active ~= t2 do
		for k,v in ipairs(active.neighbours) do
			if v.adist and v.adist < active.adist or v == t2 then
				active = v
				table.insert(st, active)
			end
		end
		count = count + 1
		if count > math.sqrt(#wt) then
			break
		end
	end
	for k,v in ipairs(wt) do
		v.adist = nil
	end
end

-- woo lua-users functions!
function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end
