-- ProtoRTS by Adrenaline Studios
-- Copyright 2009-2010 Max "Geti" Cahill, Adrenaline Studios.

--set up lua environment
require("Scripts/Create.lua")
require("Scripts/mapGen.lua")
require("Scripts/newPaddedImage.lua")
--super random seed :D
math.randomseed(os.time())
love.filesystem.setIdentity("ProtoRTS")

function love.load()

	love.graphics.setCaption( "ProtoRTS: LOADING - PLEASE WAIT" )
	love.graphics.setLineWidth(1)
	love.graphics.setFont(love.graphics.newFont("04B03.TTF", 16))

	-- Load images into tables.

	--tiles------------------------------------------------------------

	GRASS = {newPaddedImage("images/Tiles/floor_grass_1.gif"),
		newPaddedImage("images/Tiles/floor_grass_2.gif"),
		newPaddedImage("images/Tiles/floor_grass_3.gif"),
		newPaddedImage("images/Tiles/floor_grass_4.gif"),
		newPaddedImage("images/Tiles/floor_grass_5.gif"),
		newPaddedImage("images/Tiles/floor_grass_6.gif"),
		newPaddedImage("images/Tiles/floor_grass_7.gif")}
		
	HEAVY_GRASS = {newPaddedImage("images/Tiles/floor_heavygrass_1.gif"),
		newPaddedImage("images/Tiles/floor_heavygrass_2.gif"),
		newPaddedImage("images/Tiles/floor_heavygrass_3.gif"),
		newPaddedImage("images/Tiles/floor_heavygrass_4.gif"),
		newPaddedImage("images/Tiles/floor_heavygrass_5.gif"),
		newPaddedImage("images/Tiles/floor_heavygrass_6.gif")}
		
	LIGHT_GRASS = {newPaddedImage("images/Tiles/floor_lightgrass_1.gif"),
		newPaddedImage("images/Tiles/floor_lightgrass_2.gif"),
		newPaddedImage("images/Tiles/floor_lightgrass_3.gif"),
		newPaddedImage("images/Tiles/floor_lightgrass_4.gif"),
		newPaddedImage("images/Tiles/floor_lightgrass_5.gif")}
		
	PLAINS_GRASS = {newPaddedImage("images/Tiles/floor_plains_1.gif"),
		newPaddedImage("images/Tiles/floor_plains_2.gif"),
		newPaddedImage("images/Tiles/floor_plains_3.gif"),
		newPaddedImage("images/Tiles/floor_plains_4.gif"),
		newPaddedImage("images/Tiles/floor_plains_5.gif"),
		newPaddedImage("images/Tiles/floor_plains_6.gif"),
		newPaddedImage("images/Tiles/floor_plains_7.gif")}
		
	SAND = {newPaddedImage("images/Tiles/floor_sand_1.gif"),
		newPaddedImage("images/Tiles/floor_sand_2.gif"),
		newPaddedImage("images/Tiles/floor_sand_3.gif"),
		newPaddedImage("images/Tiles/floor_sand_4.gif"),
		newPaddedImage("images/Tiles/floor_sand_5.gif"),
		newPaddedImage("images/Tiles/floor_sand_6.gif"),
		newPaddedImage("images/Tiles/floor_sand_7.gif")}
		
	STONE = {newPaddedImage("images/Tiles/floor_stone_1.gif"),
		newPaddedImage("images/Tiles/floor_stone_2.gif")}
	
	WATER = {newPaddedImage("images/Tiles/floor_water_1.gif"),
		newPaddedImage("images/Tiles/floor_water_1.gif")}
	--cliffs----------------------------------------------------------------

	CLIFF_EARTH = {newPaddedImage("images/Tiles/cliff_earth_1.gif"),
		newPaddedImage("images/Tiles/cliff_earth_1.gif")}
		
	CLIFF_STONE = {newPaddedImage("images/Tiles/cliff_stone_1.gif"),
		newPaddedImage("images/Tiles/cliff_stone_2.gif")}
		
	CLIFF_SANDSTONE = {newPaddedImage("images/Tiles/cliff_sandstone_1.gif"),
		newPaddedImage("images/Tiles/cliff_sandstone_2.gif")}
		
	CLIFF_WATER = {newPaddedImage("images/Tiles/cliff_water_1.gif"),
		newPaddedImage("images/Tiles/cliff_water_1.gif")}

	-------------------------------------------------------------------------

	-- format to level_type_number.ext eg. floor_grass_1.gif, store images where appropriate.

	floormat = {
		WATER,
		SAND,
		GRASS,
		HEAVY_GRASS,
		GRASS,
		LIGHT_GRASS,
		PLAINS_GRASS,
		SAND,
		--SANDSTONE,
		STONE
	}
	cliffmat = {
		CLIFF_WATER,
		CLIFF_SANDSTONE, --sand
		CLIFF_EARTH,
		CLIFF_EARTH,
		CLIFF_EARTH,
		CLIFF_EARTH,
		CLIFF_EARTH,
		CLIFF_SANDSTONE, --sand
		--sandstone
		CLIFF_STONE
	}
	--tree declarations
	WOODS = {{newPaddedImage("images/Tiles/Vegetation/woods/trees_1_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_1_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_1_medium.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_1_dense.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_1_heavy.gif")},
		{newPaddedImage("images/Tiles/Vegetation/woods/trees_2_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_2_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_2_medium.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_2_dense.gif")},
		{newPaddedImage("images/Tiles/Vegetation/woods/trees_3_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3_medium.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3_dense.gif")},
		{newPaddedImage("images/Tiles/Vegetation/woods/trees_3_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3_medium.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_3a_dense.gif")},
		{newPaddedImage("images/Tiles/Vegetation/woods/trees_4_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_4_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_4_medium.gif"),
		newPaddedImage("images/Tiles/Vegetation/woods/trees_4_dense.gif")}}
		
	SCRUB = {{newPaddedImage("images/Tiles/Vegetation/plains/trees_1_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/plains/trees_1_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/plains/trees_1_medium.gif")},
		{newPaddedImage("images/Tiles/Vegetation/plains/trees_2_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/plains/trees_2_light.gif")},
		{newPaddedImage("images/Tiles/Vegetation/plains/trees_3_sparse.gif"),
		newPaddedImage("images/Tiles/Vegetation/plains/trees_3_light.gif"),
		newPaddedImage("images/Tiles/Vegetation/plains/trees_3_medium.gif")}}
	
	vegetationmat = {
		nil,
		SCRUB,
		WOODS,
		WOODS,
		WOODS,
		SCRUB,
		SCRUB,
		nil,
		--nil,
		nil
	}
	mineralmat = {
		{newPaddedImage("images/Tiles/Minerals/stone_large_1.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_large_2.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_large_3.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_large_4.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_large_5.gif")},
		{newPaddedImage("images/Tiles/Minerals/stone_med_1.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_med_2.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_med_3.gif")},
		{newPaddedImage("images/Tiles/Minerals/stone_small_1.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_small_2.gif"),
		newPaddedImage("images/Tiles/Minerals/stone_small_3.gif")}
	}
	buildings = {
		["HUB"] = {newPaddedImage("images/Tiles/Buildings/HUB_1.gif"),
		newPaddedImage("images/Tiles/Buildings/HUB_2.gif")},
		["HOUSE"] = {newPaddedImage("images/Tiles/Buildings/HOUSE_1.gif"),
		newPaddedImage("images/Tiles/Buildings/HOUSE_2.gif")},
		["FARM"] = {newPaddedImage("images/Tiles/Buildings/FARM_1.gif"),
		newPaddedImage("images/Tiles/Buildings/FARM_1.gif")}
	}
	units = {
		["ECO"] = {newPaddedImage("images/Tiles/Units/ecoF.gif"),
					newPaddedImage("images/Tiles/Units/ecoM.gif")}
	}
	
	--job list :D
	jobs = {
		[1] = "W", --woodcutting
		[2] = "bH", --make house
		[3] = "bF",--make farm
		[4] = "F",--use farm
		[5] = "E",--explore
		[6] = "",--social
		[7] = "R",--rest
		[8] = "",--social
		[9] = "f",--forage
		[10] = "S",--steal stuff
		[11] = "P",--patrol
	}


	--random extra stuff loading =====

	love.graphics.setBackgroundColor( 32, 64, 64 )

	BLOCK_SIZE = 32 --set the block size, in px

	--set up player 1's offset --will be at gamestart in the future
	player1ox = 0
	player1oy = 0

	--create a map --will have own menu once menu exists
	mapGen(40,40, 5)
	--timescaling
	SECONDSPERTURN = 0.25 --how long each turn takes --default 1
	TURNSPERDAY = 12 --how many "hours" per day
	DAYSPERMONTH = 30 --how many days in each month
	MONTHSPERYEAR = 2 --how many months per year.
	gametimer = 0 --set up a timer count.
	deltatimer = 0
	--------------
	CURTEAM = 1
	    
end

function love.update(dt)
	deltatimer = deltatimer + dt
	gametimer = gametimer + dt
	if dt > SECONDSPERTURN and gametimer > 10 then
		print("DESYNC")
		SECONDSPERTURN = SECONDSPERTURN * 2
	end
	while deltatimer > SECONDSPERTURN do
		for k, tile in ipairs(worldTable) do
			tile.space = tile.totalspace --get the tile space
			for k2,v in ipairs(tile.occupants) do
				tile.space = tile.space - v.space
				if v.class == "UNIT" and (v.hp <= 0 or v.lifetime >= v.life) then
					table.remove(tile.occupants, k2)
				end
			end --should be correct herein
			if math.random((#worldTable.vegetation + 1) *10) == 1 and #tile.occupants == 0 then
				createTree(tile, 1)
			end
		end
		for k, team in ipairs(teamTable) do
			team:update()
		end
		for k, unit in ipairs(worldTable.units) do
			if unit.hp < 0 then --check
				unit:destroy()
			end
		end
		for k,v in ipairs(worldTable.vegetation) do
			v:update()
		end
		deltatimer = deltatimer - SECONDSPERTURN
	end
	--key checking
	if love.keyboard.isDown("w") then
		player1oy = player1oy + 1 end
	if love.keyboard.isDown("a") then
		player1ox = player1ox + 1 end
	if love.keyboard.isDown("s") then
		player1oy = player1oy - 1 end
	if love.keyboard.isDown("d") then
		player1ox = player1ox - 1 end
	--LIMIT THE VIEW
	if player1ox < -worldSize.limits[1] then
		player1ox = -worldSize.limits[1]
	elseif player1ox > worldSize.limits[1] then
		player1ox = worldSize.limits[1] end
	if player1oy < -worldSize.limits[2] then
		player1oy = -worldSize.limits[2]
	elseif player1oy > 0 then
		player1oy = 0 end
end

function love.draw()
	--todo: 2 players (steps at a time plz)
    local sx, sy = love.graphics.getWidth() / 2, love.graphics.getHeight() / 2
	ox, oy = player1ox * BLOCK_SIZE, player1oy * BLOCK_SIZE
	
	love.graphics.setColorMode( "modulate" )
	for k,tile in ipairs(worldTable) do
		local tx = ox + sx
		local ty = oy + sy
		tx = tx + ( ( tile.pos[2] - tile.pos[1] )  * ( BLOCK_SIZE / 2 ) )
		ty = ty + ( ( tile.pos[1] + tile.pos[2] ) * ( BLOCK_SIZE / 4 ) ) + tile.alt
		if tx > -BLOCK_SIZE and tx < love.graphics.getWidth() and ty > -BLOCK_SIZE and ty < love.graphics.getHeight() then
			love.graphics.setColor( tile.colour, tile.colour, tile.colour, 255 ) -- for now, static "darking"
			if cliffmat[ round(tile.type) ][ round(tile.ckind) ] then --error negation
				for i = tile.cliffnum, 1, -1 do
					love.graphics.draw( cliffmat[ round(tile.type) ][ round(tile.ckind) ], tx, ty + ((i - 1) * (BLOCK_SIZE / 2)) ) end end --cliff printing
			if floormat[ round(tile.type) ][ round(tile.kind) ] then --error negation
				love.graphics.draw( floormat[ round(tile.type) ][ round(tile.kind) ], tx, ty) end --tile printing
			for k2,v in ipairs(tile.occupants) do
				if v.class == "RESOURCE_TREE" then
					if vegetationmat[tile.type] and vegetationmat[tile.type][ round(v.sprite) ] then --double checking for no errors :D
						if vegetationmat[tile.type][ round(v.sprite) ][ round(v.size) ] then
							love.graphics.draw( vegetationmat[tile.type][ round(v.sprite) ][ round(v.size) ], tx - 16, ty - 80)
						else
							--print("SPRITE: "..v.sprite.." | SIZE: "..v.size.." | TILETYPE: "..tile.type.." | POS: "..tile.pos[1]..", "..tile.pos[2].." ERROR")
							v.sprite = math.random(#vegetationmat[tile.type])
							v.size = 1 --reset the tree, look for problems
						end
					end
				end
				if v.class == "RESOURCE_MINERAL" then
					love.graphics.draw( mineralmat[ round(v.size) ][ round(v.sprite) ], tx - 16, ty - 48)
				end
				if v.class == "BUILDING" then
					love.graphics.draw( buildings[ v.name ][ round(v.sprite) ], tx - 16, ty - 80)
				end
				if v.class == "UNIT" then
					if v.team == CURTEAM then
						love.graphics.setColor( tile.colour , tile.colour * math.max(0.7, v.hp / v.maxhp), tile.colour * math.max(0.5, v.hp / v.maxhp), (1 - (v.lifetime / v.life)) * 210 + 45 ) -- for now, static "darking"
					else --ENEMY!
						love.graphics.setColor( 255 * (tile.colour/255) , 128 * (tile.colour/255), 128 * (tile.colour/255), 255 ) -- for now, static "darking"
					end
					local row = k2 - 1
					local column = 0
					while row > 3 do
						column = column + 1
						row = row - 4
					end
					local x = ( column - row ) * 4
					local y = ( row - column ) * 2
					love.graphics.draw( units[ v.type ][v.gender], tx + x, ty - 16 + y )
					love.graphics.setColor( tile.colour, tile.colour, tile.colour, 255 ) -- put the colour back the way you found it
					--love.graphics.print(v.job,tx,ty- 10) --job printing
				end
			end
			--love.graphics.print(tile.type,tx + 12,ty) --type printing
		end
	end
	love.graphics.setColor( 0, 0, 0, 128 )
	love.graphics.rectangle( "fill", 0, 0, love.graphics.getWidth(), 17 ) 
	for k,v in ipairs(teamTable) do
		updateTeamString(v)
		love.graphics.setColor( 0, 0, 0, 128 )
		love.graphics.rectangle( "fill", 0, 17 * k, love.graphics.getWidth(), 17 ) 
		love.graphics.setColor( 255, 255, 255, 255 )
		if #v.units > 0 then
			love.graphics.print("Team "..k..v.string, 0, 17 * k - 2)
		else
			love.graphics.print("Team "..k.." ALL DEAD", 5, 17 * k - 2)
		end
	end
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.print("TIMESCALE: x"..(1/SECONDSPERTURN), 5, (#teamTable + 1) * 17 - 2)
	love.graphics.print("CURRENT TEAM: "..CURTEAM, 200, (#teamTable + 1) * 17 - 2)
	love.graphics.setCaption( "ProtoRTS [".. love.timer.getFPS() .." Fps]".."[Delta: "..round(love.timer.getDelta(), 4).."]" )
end

function love.keyreleased(k)
	
end

function love.keypressed(k)
	--team switching
	if k == "q" then
		CURTEAM = CURTEAM - 1
		if CURTEAM < 1 then CURTEAM = #teamTable end
		end
	if k == "e" then
		CURTEAM = CURTEAM + 1
		if CURTEAM > #teamTable then CURTEAM = 1 end
		end
	--job giving
	if k == "r" then --WOOD
		table.insert(teamTable[CURTEAM].tasks, 1)
		end
	if k == "f" then --FOOD
		table.insert(teamTable[CURTEAM].tasks, 4)
		end
	if k == "t" then --HOUSE
		table.insert(teamTable[CURTEAM].tasks, 2)
		end
	if k == "g" then --FARM
		table.insert(teamTable[CURTEAM].tasks, 3)
		end
	if k == "z" then --REST
		table.insert(teamTable[CURTEAM].tasks, 7)
		end
	if k == "x" then --SOCIAL
		table.insert(teamTable[CURTEAM].tasks, math.random(6,8))
		end
	if k == "c" then --EXPLORE
		table.insert(teamTable[CURTEAM].tasks, 5)
		end
	if k == "v" then --PATROL
		table.insert(teamTable[CURTEAM].tasks, 11)
		end
	if k == "b" then --ATTACK
		local self = teamTable[CURTEAM];
		local attack = self; while attack == self do
		attack = teamTable[math.random(#teamTable)] end --get a random team to attack
		print("TEAM "..self.number.." IS ATTACKING TEAM "..attack.number)
		for k,v in ipairs(self.units) do
			--all go there
			pathTo(v.tile, attack.buildings[1].tile, worldTable, v.toMove, v)
		end end
	--timescaling
	if k =="1" then
		SECONDSPERTURN = SECONDSPERTURN * 2 end
	if k == "2" then
		SECONDSPERTURN = SECONDSPERTURN / 2 end
	--system stuff
	if k == "f3" then
		love.load()
	end
	if k == "escape" then
		love.event.push("q")
	end
end
